@@ -16,6 +16,7 @@ import pprint
 import random
 import re
 import shlex
+import socket
 import string
 import subprocess
 import sys
@@ -53,11 +54,11 @@ def logline(*args):
     msg = '%s %s' % (cstr(caller, 'yellow'), args[0])
     logging.warning(msg, *args[1:])
 
-def shell(cmd, cwd=None, env=None, decode_utf8=True):
+def shell(cmd, cwd=None, env=None, decode_utf8=True, timeout=None):
     if type(cmd) not in (tuple, list):
         cmd = shlex.split(cmd)
     proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False, cwd=cwd, env=env)
-    stdout, stderr = proc.communicate()
+    stdout, stderr = proc.communicate(timeout=timeout)
     if decode_utf8:
         stdout = stdout.decode('utf-8').strip()
         stderr = stderr.decode('utf-8').strip()
@@ -883,7 +884,7 @@ def incr_stop(obj, timer_name):
 # Support for MaxMind's free GeoLite2 database, sign up at https://www.maxmind.com/en/geolite2/signup
 # Add geoip2 to setup.py and pip requirements.txt, and set geolite_db_path to the filesystem location of the downloaded db in application config.py
 class GeoLite(object):
-    geolite = None
+    geolite_db = None
     geolite_path = None
 
     def __init__(self, geolite_path):
@@ -892,17 +893,39 @@ class GeoLite(object):
 
     @classmethod
     def get_ip(cls, ip_addr):
+        if not ip_addr:
+            return {}
         # Only open the database once, and only as-needed
         if not os.path.exists(cls.geolite_path):
             logging.error("GeoLite db not found: %s" % cls.geolite_path)
             return {}
         import geoip2.database
-        if not cls.geolite:
-            cls.geolite = geoip2.database.Reader(cls.geolite_path)
-        record = cls.geolite.city(ip_addr)
+        if not cls.geolite_db:
+            cls.geolite_db = geoip2.database.Reader(cls.geolite_path)
+        try:
+            record = cls.geolite_db.city(ip_addr)
+        except geoip2.errors.AddressNotFoundError as ex:
+            logging.exception(ex)
+            return {}
         geoip = {'continent': record.continent.name, 'country_iso_code': record.country.iso_code, 'country': record.country.name, 'time_zone': record.location.time_zone,
                  'city': record.city.name, 'ip_addr': ip_addr}
         if len(record.subdivisions):
             geoip['region_iso_code'] = record.subdivisions[0].iso_code
             geoip['region'] = record.subdivisions[0].name
         return geoip
+
+
+def monitor_shell(cmd, timeout=8):
+    # Run the monitoring command with 8 second timeout. Monitoring should fail fast.
+    try:
+        stdout, stderr, exit_status = shell(cmd, timeout=timeout)
+    except subprocess.TimeoutExpired as ex:
+        logging.exception(ex)
+        stdout = ""
+        stderr = str(ex)
+        exit_status = 1
+    # Write it to the db
+    import aegis.model
+    monitor = {'monitor_host': socket.gethostname(), 'monitor_cmd': cmd, 'monitor_stdout': stdout, 'monitor_stderr': stderr, 'monitor_status': exit_status}
+    monitor_id = aegis.model.Monitor.insert_columns(**monitor)
+    return exit_status == 0
